/**
 * Utility functions to ease the writing of action (pre/post
 * installation/removal) scripts.
 *
 * Note on exception handling:
 * All exceptions are expected to be caught and handled by the caller.
 *
 * Note on implementation:
 * This is not implemented as a custom object because Common Installer under
 * GUI (but not console) mode (as of 5 May 2005) does not support custom
 * objects.
 */

/**
 * Read the contents of a file.
 *
 * @param filename (String) The full path of the file to read from.
 * @return (String) Contents of the file.
 * @throws Exception if an error occurs.
 */
function readFromFile( filename )
{
	var fileContents = "";
	filename = transformPathSeparators( filename );

	try
	{
		// Read the file.
		log( 2, "Reading file: " + filename );

		// Create and initialise the file.
		var file = Components.classes["@mozilla.org/file/local;1"].
			createInstance( Components.interfaces.nsILocalFile );
		// NOTE: This will not work on Mac!
		file.initWithPath( filename );

		// Use IOService to convert the file to a file URI.
		var ioService =
			Components.classes["@mozilla.org/network/io-service;1"].
			getService( Components.interfaces.nsIIOService );
		var fileUri = ioService.newFileURI( file.QueryInterface(
			Components.interfaces.nsIFile ) );

		// Get a channel from the URI.
		var channel = ioService.newChannelFromURI( fileUri );

		// Create an input stream.
		var inStream =
			Components.classes["@mozilla.org/scriptableinputstream;1"].
			createInstance( Components.interfaces.nsIScriptableInputStream );

		try
		{
			inStream.init( channel.open() );

			var bytes;
			while ( (bytes = inStream.available()) > 0 )
			{
				fileContents += inStream.read( bytes );
			}
		}
		finally
		{
			inStream.close();
		}

		log( 3, "Read completed." );
	}
	catch ( e )
	{
		throw "Caught exception: [ActionUtils.readFromFile()] " + e;
	}

	return fileContents;
}

/**
 * Backup a file by appending ".bak" to the filename. If the backup file already
 * exists, it will be replaced with the new backup file.
 *
 * @param filename (String) The full path of the file to backup.
 * @throws Exception if an error occurs.
 */
function backupFile( filename )
{
	filename = transformPathSeparators( filename );
	try
	{
		log( 2, "Backing up " + filename );

		// Create and initialise the file.
		var file = Components.classes["@mozilla.org/file/local;1"].
			createInstance( Components.interfaces.nsILocalFile );
		file.initWithPath( filename );
		file.moveTo( file.parent, file.leafName + ".bak" );

		log( 3, "Backup completed." );
	}
	catch ( e )
	{
		throw "Caught exception: [ActionUtils.backupFile()] " + e;
	}
}

/**
 * Write content to a file. The file will have 664 permissions.
 *
 * @param filename (String) The full path of the file to write to.
 * @param content (String) The content to write.
 * @param backup (boolean) If true and if [filename] already exists, [filename]
 *        will be renamed to [filename].bak before writing the new content to
 *        [filename]. If false and if [filename] already exists, [filename] will
 *        be deleted before writing the new content to [filename].
 * @throws Exception if an error occurs.
 */
function writeToFile( filename, content, backup )
{
	filename = transformPathSeparators( filename );

	try
	{
		log( 2, "Writing to file: " + filename );

		// Create and initialise the file.
		var file = Components.classes["@mozilla.org/file/local;1"].
			createInstance( Components.interfaces.nsILocalFile );
		file.initWithPath( filename );

		if ( file.exists() )
		{
			if ( backup )
			{
				backupFile( filename );
			}
			else
			{
				file.remove( false );
			}
		}

		// To write to the file, make an output stream, initialize the
		// stream with the file, and write to it.
		var outStream =
			Components.classes["@mozilla.org/network/file-output-stream;1"].
			createInstance( Components.interfaces.nsIFileOutputStream );

		try
		{
			outStream.init( file.QueryInterface(
				Components.interfaces.nsIFile ), 0x04 | 0x08, 0664, 0 );
			// octal 0664 gives "664" permissions.
			var result = outStream.write( content, content.length );
		}
		finally
		{
			outStream.close();
		}

		log( 3, "Write completed." );
	}
	catch ( e )
	{
		throw "Caught exception: [ActionUtils.writeToFile()] " + e;
	}
}

/**
 * Create folders/directories. The folder will have 775 permissions. Does
 * nothing if the folder already exists.
 *
 * @param foldername (String) The full path of the folder to create.
 * @throws Exception if an error occurs.
 */
function createFolders( foldername )
{
	foldername = transformPathSeparators( foldername );
	try
	{
		var localFile = Components.classes['@mozilla.org/file/local;1'].
			createInstance( Components.interfaces.nsILocalFile );
		localFile.initWithPath( foldername );

		if ( localFile.exists() )
		{
			log( 2, foldername + " already exists." );
			return;
		}

		localFile.create( localFile.DIRECTORY_TYPE, 0775 );
	}
	catch ( e )
	{
		throw "Caught exception: [ActionUtils.createFolders()] " + e;
	}
}

/**
 * Remove a file/directory. Will not do anything if the file/directory does not
 * exist.
 *
 * @param filename (String) File/directory name.
 * @param recursive (boolean) If true, remove a non-empty directory recursively.
 * @throws Exception if an error occurs.
 */
function removeFile( filename, recursive )
{
	filename = transformPathSeparators( filename );
	try
	{
		log( 2, "Removing file/directory: " + filename );

		var localFile = Components.classes['@mozilla.org/file/local;1'].
			createInstance( Components.interfaces.nsILocalFile );
		localFile.initWithPath( filename );

		if ( !localFile.exists() )
		{
			log( 1, filename + " does not exist." );
			return;
		}
		else
		{
			localFile.remove( recursive );
		}

		log( 3, "File/directory removal completed." );
	}
	catch ( e )
	{
		throw "Caught exception: [ActionUtils.removeFile()] " + e;
	}
}

/**
 * Get $NCHOME.
 *
 * @return (String) The $NCHOME path.
 * @throws Exception if an error occurs.
 */
function getNchome()
{
	var nchome = "";

	try
	{
		var propMgr = Components.classes["@micromuse.com/NCIProperties;1"].
			getService( Components.interfaces.nsINCIProperties );

		nchome = propMgr.GetProp( "NCHOME.INSTALLDIR" ).value;
	}
	catch ( e )
	{
		throw "Caught exception: [ActionUtils.getNchome()] " + e;
	}

	return nchome;
}

/**
 * Log a message.
 *
 * @param level (int) The level to log with. Valid levels are: 0 - Error,
 *		  1 - Warning 2 - Info, 3 - Debug.
 * @param msg (String) The message to log.
 */
function log( level, msg )
{
	try
	{
		var logger = Components.classes["@micromuse.com/NCILog;1"].
			getService( Components.interfaces.nsINCILog );

		switch ( level )
		{
			case 0: logger.error( msg ); break;
			case 1: logger.warning( msg ); break;
			case 2: logger.info( msg ); break;
			case 3: logger.debug( msg ); break;
			default: break;
		}
	}
	catch ( e )
	{
		// Logging failure must not affect functionality, so fail silently.
	}
}

/**
 * Translate a filename so that it uses the correct path separator based on the
 * local platform.
 *
 * @param filename (String) The supplied filename
 *
 * @return filename (String) The file path updated with the correct path separators.
 */
function transformPathSeparators( filename )
{
	var newContents;
	if( ncipackage.shortArch == "win32" )
	{
		// Replace ObjectServer port.
		regex = /\//g;
		newContents = filename.replace( regex,
			"\\" );
	}
	else
	{
		regex = /\\/g;
		newContents = filename.replace( regex,
			"\/" );
	}
	return newContents;
}
